@@ -1,4 +1,6 @@
+import traceback
 from enum import Enum
+from typing import List, Union
 
 from minidump.minidumpfile import *
 from unicorn import *
@@ -7,6 +9,7 @@ from pefile import *
 import inspect
 from .native import *
 from capstone import *
+from collections import OrderedDict
 
 syscall_functions = {}
 
@@ -32,6 +35,8 @@ class GDT_FLAGS:
 
 
 def map_unicorn_perms(protect: AllocationProtect):
+    if isinstance(protect, int):
+        protect = AllocationProtect(protect)
     mapping = {
         AllocationProtect.PAGE_EXECUTE: UC_PROT_EXEC | UC_PROT_READ,
         AllocationProtect.PAGE_EXECUTE_READ: UC_PROT_EXEC | UC_PROT_READ,
@@ -90,7 +95,6 @@ class Registers:
             "rdi": UC_X86_REG_RDI,
             "rdx": UC_X86_REG_RDX,
             "rip": UC_X86_REG_RIP,
-            "riz": UC_X86_REG_RIZ,
             "rsi": UC_X86_REG_RSI,
             "rsp": UC_X86_REG_RSP,
             "si": UC_X86_REG_SI,
@@ -103,17 +107,7 @@ class Registers:
             "cr2": UC_X86_REG_CR2,
             "cr3": UC_X86_REG_CR3,
             "cr4": UC_X86_REG_CR4,
-            "cr5": UC_X86_REG_CR5,
-            "cr6": UC_X86_REG_CR6,
-            "cr7": UC_X86_REG_CR7,
             "cr8": UC_X86_REG_CR8,
-            "cr9": UC_X86_REG_CR9,
-            "cr10": UC_X86_REG_CR10,
-            "cr11": UC_X86_REG_CR11,
-            "cr12": UC_X86_REG_CR12,
-            "cr13": UC_X86_REG_CR13,
-            "cr14": UC_X86_REG_CR14,
-            "cr15": UC_X86_REG_CR15,
             "dr0": UC_X86_REG_DR0,
             "dr1": UC_X86_REG_DR1,
             "dr2": UC_X86_REG_DR2,
@@ -122,14 +116,6 @@ class Registers:
             "dr5": UC_X86_REG_DR5,
             "dr6": UC_X86_REG_DR6,
             "dr7": UC_X86_REG_DR7,
-            "dr8": UC_X86_REG_DR8,
-            "dr9": UC_X86_REG_DR9,
-            "dr10": UC_X86_REG_DR10,
-            "dr11": UC_X86_REG_DR11,
-            "dr12": UC_X86_REG_DR12,
-            "dr13": UC_X86_REG_DR13,
-            "dr14": UC_X86_REG_DR14,
-            "dr15": UC_X86_REG_DR15,
             "fp0": UC_X86_REG_FP0,
             "fp1": UC_X86_REG_FP1,
             "fp2": UC_X86_REG_FP2,
@@ -300,8 +286,35 @@ class Registers:
             "mxcsr": UC_X86_REG_MXCSR,
             "fs_base": UC_X86_REG_FS_BASE,
             "gs_base": UC_X86_REG_GS_BASE,
-            "rflags": UC_X86_REG_EFLAGS,
         }
+        if unicorn.__version__[0] < '2':
+            self._regmap.update({
+                "riz": UC_X86_REG_RIZ,
+                "cr5": UC_X86_REG_CR5,
+                "cr6": UC_X86_REG_CR6,
+                "cr7": UC_X86_REG_CR7,
+                "cr9": UC_X86_REG_CR9,
+                "cr10": UC_X86_REG_CR10,
+                "cr11": UC_X86_REG_CR11,
+                "cr12": UC_X86_REG_CR12,
+                "cr13": UC_X86_REG_CR13,
+                "cr14": UC_X86_REG_CR14,
+                "cr15": UC_X86_REG_CR15,
+                "dr8": UC_X86_REG_DR8,
+                "dr9": UC_X86_REG_DR9,
+                "dr10": UC_X86_REG_DR10,
+                "dr11": UC_X86_REG_DR11,
+                "dr12": UC_X86_REG_DR12,
+                "dr13": UC_X86_REG_DR13,
+                "dr14": UC_X86_REG_DR14,
+                "dr15": UC_X86_REG_DR15,
+                "rflags": UC_X86_REG_EFLAGS,
+            })
+        else:
+            self._regmap.update({
+                "flags": UC_X86_REG_FLAGS,
+                "rflags": UC_X86_REG_RFLAGS
+            })
         if self._x64:
             self._regmap.update({
                 "cax": UC_X86_REG_RAX,
@@ -382,10 +395,11 @@ class Arguments:
             raise Exception("not implemented!")
 
 
-class Dumpulator:
+class Dumpulator(Architecture):
     def __init__(self, minidump_file, trace=False):
         self._minidump = MinidumpFile.parse(minidump_file)
-        self._x64 = type(self._minidump.threads.threads[0].ContextObject) is not WOW64_CONTEXT
+        super().__init__(type(self._minidump.threads.threads[0].ContextObject) is not WOW64_CONTEXT)
+        self.addr_mask = 0xFFFFFFFFFFFFFFFF if self._x64 else 0xFFFFFFFF
 
         if trace:
             self.trace = open(minidump_file + ".trace", "w")
@@ -412,7 +426,6 @@ class Dumpulator:
         self._setup_syscalls()
         self.exports = self._setup_exports()
 
-
     # Source: https://github.com/mandiant/speakeasy/blob/767edd2272510a5badbab89c5f35d43a94041378/speakeasy/windows/winemu.py#L533
     def _setup_gdt(self, teb_addr):
         """
@@ -515,20 +528,22 @@ class Dumpulator:
 
         info: MinidumpMemoryInfo
         for info in self._minidump.memory_info.infos:
+            emu_addr = info.BaseAddress & self.addr_mask
             if info.State == MemoryState.MEM_COMMIT:
-                print(f"mapped base: 0x{info.BaseAddress:x}, size: 0x{info.RegionSize:x}, protect: {info.Protect}")
-                self._uc.mem_map(info.BaseAddress, info.RegionSize, map_unicorn_perms(info.Protect))
-            elif info.State == MemoryState.MEM_FREE and info.BaseAddress > 0x10000 and info.RegionSize >= self._allocate_size:
-                self._allocate_base = info.BaseAddress
+                print(f"mapped base: 0x{emu_addr:x}, size: 0x{info.RegionSize:x}, protect: {info.Protect}")
+                self._uc.mem_map(emu_addr, info.RegionSize, map_unicorn_perms(info.Protect))
+            elif info.State == MemoryState.MEM_FREE and emu_addr > 0x10000 and info.RegionSize >= self._allocate_size:
+                self._allocate_base = emu_addr
 
         memory = self._minidump.get_reader().get_buffered_reader()
         seg: MinidumpMemorySegment
         for seg in self._minidump.memory_segments_64.memory_segments:
-            print(f"initialize base: 0x{seg.start_virtual_address:x}, size: 0x{seg.size:x}")
+            emu_addr = seg.start_virtual_address & self.addr_mask
+            print(f"initialize base: 0x{emu_addr:x}, size: 0x{seg.size:x}")
             memory.move(seg.start_virtual_address)
             assert memory.current_position == seg.start_virtual_address
             data = memory.read(seg.size)
-            self._uc.mem_write(seg.start_virtual_address, data)
+            self._uc.mem_write(emu_addr, data)
 
         thread = self._minidump.threads.threads[0]
         if self._x64:
@@ -577,7 +592,13 @@ class Dumpulator:
             self.regs.edi = context.Edi
             self.regs.eip = context.Eip
 
-        self._setup_gdt(thread.Teb)
+        # Remove the trap flag
+        self.regs.eflags &= ~0x100
+
+        # Set up TEB
+        teb = thread.Teb & 0xFFFFFFFFFFFFF000
+        self._setup_gdt(teb)
+        self.regs.gs_base = teb
 
     def _setup_exports(self):
         exports = {}
@@ -648,9 +669,6 @@ class Dumpulator:
                 argcount = len(argspec.args) - 1
             self.syscalls.append((name, cb, argcount))
 
-    def ptr_size(self):
-        return 8 if self._x64 else 4
-
     def push(self, value):
         csp = self.regs.csp - self.ptr_size()
         self.write_ptr(csp, value)
@@ -660,27 +678,17 @@ class Dumpulator:
         return self._uc.mem_read(addr, size)
 
     def write(self, addr, data):
-        return self._uc.mem_write(addr, data)
+        self._uc.mem_write(addr, data)
 
-    def read_ptr(self, addr):
-        return struct.unpack("<Q" if self._x64 else "<I", self.read(addr, self.ptr_size()))[0]
+    def protect(self, addr, size, protect):
+        perms = map_unicorn_perms(protect)
+        self._uc.mem_protect(addr, size, perms)
 
-    def read_ulong(self, addr):
-        return struct.unpack("<I", self.read(addr, 4))[0]
+    def call(self, addr, args: List[int]=[], regs: dict={}, count=0):
+        # allow passing custom registers
+        for name, value in regs.items():
+            self.regs.__setattr__(name, value)
 
-    def read_long(self, addr):
-        return struct.unpack("<i", self.read(addr, 4))[0]
-
-    def write_ulong(self, addr, value):
-        return self._uc.mem_write(addr, struct.pack("<I", value))
-
-    def write_long(self, addr, value):
-        return self._uc.mem_write(addr, struct.pack("<i", value))
-
-    def write_ptr(self, addr, value):
-        return self._uc.mem_write(addr, struct.pack("<Q" if self._x64 else "<I", value))
-
-    def call(self, addr, args, count=0):
         # set up arguments
         if self._x64:
             for index, value in enumerate(args):
@@ -694,13 +702,6 @@ class Dumpulator:
         self.start(addr, end=CAVE_ADDR, count=count)
         return self.regs.cax
 
-    def read_str(self, addr, encoding="utf-8"):
-        data = self.read(addr, 512)
-        nullidx = data.find(b'\0')
-        if nullidx != -1:
-            data = data[:nullidx]
-        return data.decode(encoding)
-
     def allocate(self, size):
         if not self._allocate_ptr:
             self._uc.mem_map(self._allocate_base, self._allocate_size)
@@ -743,19 +744,19 @@ def _hook_mem(uc: Uc, access, address, size, value, dp: Dumpulator):
     return False
 
 def _get_regs(instr):
-    regs = set()
+    regs = OrderedDict()
     for op in instr.operands:
         if op.type == CS_OP_REG:
-            regs.add(instr.reg_name(op.value.reg))
+            regs[instr.reg_name(op.value.reg)] = None
         elif op.type == CS_OP_MEM:
             if op.value.mem.base != 0:
-                regs.add(instr.reg_name(op.value.mem.base))
+                regs[instr.reg_name(op.value.mem.base)] = None
             if op.value.mem.index != 0:
-                regs.add(instr.reg_name(op.value.mem.index))
+                regs[instr.reg_name(op.value.mem.index)] = None
     for reg in instr.regs_read:
-        regs.add(instr.reg_name(reg))
+        regs[instr.reg_name(reg)] = None
     for reg in instr.regs_write:
-        regs.add(instr.reg_name(reg))
+        regs[instr.reg_name(reg)] = None
     return regs
 
 def _hook_code(uc: Uc, address, size, dp: Dumpulator):
@@ -823,7 +824,7 @@ def _hook_syscall(uc: Uc, dp: Dumpulator):
                 argname = argspec.args[1 + i]
                 argtype = argspec.annotations[argname]
                 if issubclass(argtype, PVOID):
-                    argvalue = argtype(dp.args[i], dp.read)
+                    argvalue = argtype(dp.args[i], dp)
                 else:
                     argvalue = argtype(dp.args[i])
                 args.append(argvalue)
@@ -834,9 +835,15 @@ def _hook_syscall(uc: Uc, dp: Dumpulator):
 
                 print(f"    {_arg_type_string(argvalue)} {argname} = {_arg_to_string(argvalue)}{comma}")
             print(")")
-            status = cb(dp, *args)
-            print(f"status = {status:x}")
-            dp.regs.cax = status
+            try:
+                status = cb(dp, *args)
+                print(f"status = {status:x}")
+                dp.regs.cax = status
+            except Exception as exc:
+                sys.stderr = sys.stdout
+                traceback.print_exception(type(exc), exc, exc.__traceback__)
+                print(f"Exception thrown during syscall implementation, stopping emulation!")
+                uc.emu_stop()
         else:
             print(f"syscall index: {index:0x} -> {name} not implemented!")
             uc.emu_stop()
