@@ -7,7 +7,10 @@ from .native import *
 
 
 def syscall(func):
-    syscall_functions[func.__name__] = func
+    name: str = func.__name__
+    if name.startswith("Nt"):
+        name = "Zw" + name[2:]
+    syscall_functions[name] = func
     return func
 
 
@@ -128,4 +131,31 @@ def ZwQueryVolumeInformationFile(dp: Dumpulator,
                                  FsInformationClass: FS_INFORMATION_CLASS
                                  ):
     # TODO: implement
-    return STATUS_SUCCESS
\ No newline at end of file
+    return STATUS_SUCCESS
+
+@syscall
+def ZwQueryInformationToken(dp: Dumpulator):
+    return STATUS_NOT_IMPLEMENTED
+
+@syscall
+def ZwDisplayString(dp: Dumpulator,
+                    String: P(UNICODE_STRING)
+                    ):
+    print("debug: " + String.read_unicode_str())
+    return STATUS_PRIVILEGE_NOT_HELD
+
+@syscall
+def ZwProtectVirtualMemory(dp: Dumpulator,
+                           ProcessHandle: HANDLE,
+                           BaseAddress: P(PVOID),
+                           RegionSize: P(ULONG),
+                           NewProtect: ULONG,
+                           OldProtect: P(ULONG)
+                           ):
+    base = BaseAddress[0] & 0xFFFFFFFFFFFFF000
+    size = round_to_pages(RegionSize[0])
+
+    print(f"protect {base:x}[{size:x}] = {NewProtect:x}")
+    dp.protect(base, size, NewProtect)
+    # TODO: OldProtect is not implemented
+    return STATUS_SUCCESS
